import time
import os
import utilum
from . import scripts


def gitConfig(name, email, repoPath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''
    utilum.system.shell(wrapperRegular(
        f'git config --local user.name "{name}"'))
    utilum.system.shell(wrapperRegular(
        f'git config --local user.email "{email}"'))


def gitInitOrRegular(repoPath, groupRepoPath, tokenFilePath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''

    # 1/6
    cmd1 = wrapperRegular(f'''git init --initial-branch=main''')
    utilum.system.shell(cmd1)

    # 2/6
    dstToken = utilum.file.readFile(tokenFilePath).replace("\n", "")
    rUrl = f'https://gitlab-ci-token:{dstToken}@gitlab.com/{groupRepoPath}.git'
    cmd2 = wrapperRegular(f'''git remote set-url origin {rUrl}''')
    utilum.system.shell(cmd2)

    # 3-5/6
    cmd3 = wrapperRegular(f'''git add .''')
    cmd4 = wrapperRegular(f'''git commit -m "Regular Update"''')
    cmd5 = wrapperRegular(f'''git push origin main''')

    utilum.system.shell(cmd3)
    utilum.system.shell(cmd4)
    utilum.system.shell(cmd5)

    # 6/6
    rUrlClean = f'https://gitlab.com/{groupRepoPath}.git'
    cmd6 = wrapperRegular(f'''git remote set-url origin {rUrlClean}''')
    utilum.system.shell(cmd6)

    return None


def manageDatabases(repoPath, containerName, passwordFilePath):
    cmd1 = scripts.showDatabasesDocker(containerName, passwordFilePath)
    (out, err) = utilum.system.shellRead(cmd1)
    decoded = out.decode('utf-8')
    dbs = decoded.split("\n")
    dbs = dbs[1:-1]

    for databaseName in dbs:
        outputDbPath = os.path.join(repoPath, databaseName + '.sql')
        exportCmd = scripts.exportDatabaseDocker(
            containerName, passwordFilePath, databaseName, outputDbPath)
        utilum.system.shell(exportCmd)


def flow(config):
    # Mid Function to Transfer DB Files
    manageDatabases(config.STAGE_STORAGE_PATH,
                    config.CONTAINER_NAME, config.PASSWORD_FILE_PATH)

    # git config set
    gitConfig(config.GIT_NAME, config.GIT_EMAIL, config.STAGE_STORAGE_PATH)

    # Last Function to Commit
    gitInitOrRegular(config.STAGE_STORAGE_PATH, config.GROUP_REPO_PATH, config.TOKEN_FILE_PATH)


def start(config):
    count = 0.001
    while (True):
        print("\nCount: ", count)
        flow(config)
        time.sleep(config.INTERVAL)
        count += 0.001



import time
import utilum
from . import scripts


def gitConfig(name, email, repoPath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''
    utilum.system.shell(wrapperRegular(
        f'git config --local user.name "{name}"'))
    utilum.system.shell(wrapperRegular(
        f'git config --local user.email "{email}"'))


def gitInitOrRegular(repoPath, message, branch):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''

    cmd3 = wrapperRegular(f'''git add .''')
    cmd4 = wrapperRegular(f'''git commit -m "{message}"''')
    cmd5 = wrapperRegular(f'''git push origin {branch}''')

    utilum.system.shell(cmd3)
    utilum.system.shell(cmd4)
    utilum.system.shell(cmd5)

    return None


def manageDatabases(repoPath, containerName):
    cmd1 = scripts.createMongoDbBackup(containerName)
    (out1, err1) = utilum.system.shellRead(cmd1)

    cmd2 = scripts.removeMongoDbBackup(repoPath)
    (out2, err2) = utilum.system.shellRead(cmd2)

    cmd3 = scripts.copyMongoDbBackupFiles(containerName, repoPath)
    (out3, err3) = utilum.system.shellRead(cmd3)

    return


def flow(config):
    # Mid Function to Transfer DB Files
    manageDatabases(config.STAGE_STORAGE_PATH,
                    config.CONTAINER_NAME)

    # git config set
    gitConfig(config.GIT_NAME, config.GIT_EMAIL, config.STAGE_STORAGE_PATH)

    # Last Function to Commit
    gitInitOrRegular(config.STAGE_STORAGE_PATH,
                     config.GIT_MESSAGE, config.GIT_BRANCH)


def start(config):
    count = 0.001
    while (True):
        print("\nCount: ", count)
        flow(config)
        time.sleep(config.INTERVAL)
        count += 0.001



import time
import os
import utilum
from . import scripts


def gitConfig(name, email, repoPath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''
    utilum.system.shell(wrapperRegular(
        f'git config --local user.name "{name}"'))
    utilum.system.shell(wrapperRegular(
        f'git config --local user.email "{email}"'))


def gitAdd(filePath, repoPath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''

    cmd3 = wrapperRegular(f'''git add "{filePath}"''')
    utilum.system.shell(cmd3)


def gitCommit(repoPath, message, branch):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''

    cmd4 = wrapperRegular(f'''git commit -m "{message}"''')
    cmd5 = wrapperRegular(f'''git push origin {branch}''')

    utilum.system.shell(cmd4)
    utilum.system.shell(cmd5)

    return None


MB = 1000*1000

def manageDs(config, srcPath, repoPath, MAX_SIZE):
    cmd = scripts.removeDsBackupFiles(repoPath)
    (out, err) = utilum.system.shellRead(cmd)

    cmd2 = scripts.copyDsBackupFiles(srcPath, repoPath)
    (out2, err2) = utilum.system.shellRead(cmd2)

    commitPath = os.path.join(repoPath, 'dump')

    gBatches = []
    tBatches = []
    batches = []
    batchSize = 0
    for file in utilum.file.folderTraversal(commitPath):
        size = utilum.file.getSize(file)/(MB)
        
        if((batchSize + size) > MAX_SIZE):
            gBatches.append([batchSize, batches])
            batchSize = size
            batches = [file]
        else:
            batchSize += size
            batches.append(file)

        tBatches = [batchSize, batches]
        
    gBatches.append(tBatches)
    for igb, gb in enumerate(gBatches):
        for dFile in gb[1]:
            gitAdd(dFile, repoPath)
        gitCommit(repoPath, config.GIT_MESSAGE, config.GIT_BRANCH)
        print("*********************------------------------------------------------------*********************")
        print("[Batch/Total][Size]: ", f"[{igb+1}/{len(gBatches)}][{gb[0]}]")
        print("*********************------------------------------------------------------*********************")

    return


def flow(config):
    # Mid Function to Transfer DB Files
    manageDs(config, config.SRC_PATH, config.DST_PATH, config.GIT_MAX_PUSH_SIZE)

    # git config set
    gitConfig(config.GIT_NAME, config.GIT_EMAIL, config.DST_PATH)



def start(config):
    count = 0.001
    while (True):
        print("\nCount: ", count)
        flow(config)
        time.sleep(config.INTERVAL)
        count += 0.001



import os

################################################ -------------------------------################################################
# [1/4] MySQL


def showDatabases():
    return '''sudo mysql -u root -e 'show databases';'''


def exportDatabase(dbName, dbPath):
    return f'''sudo mysqldump -u root {dbName} > '{dbPath}' --skip-dump-date --extended-insert=FALSE;'''
    # return f'''sudo mysqldump -u root {dbName} > '{dbPath}' --skip-dump-date --extended-insert | sed 's/),(/),\n(/g' > '{dbPath}' ;'''


def showDatabasesDocker(containerName, passwordFilePath):
    return f'''
    PASSWORD_PATH='{passwordFilePath}'
    PASSWORD=`cat $PASSWORD_PATH`
    docker exec {containerName} mysql -u root --password=$PASSWORD -e "SHOW DATABASES";
    '''


def exportDatabaseDocker(containerName, passwordFilePath, databaseName, outputPath):
    return f'''
    PASSWORD_PATH='{passwordFilePath}'
    PASSWORD=`cat $PASSWORD_PATH`
    docker exec {containerName} /usr/bin/mysqldump -u root --password=$PASSWORD {databaseName} > {outputPath} --skip-dump-date --extended-insert=FALSE;
    '''


################################################ -------------------------------################################################
# [2/4] MongoDb

def createMongoDbBackup(containerName):
    return f'''
    docker exec {containerName} sh -c 'mongodump -o /backup'
    '''


def removeMongoDbBackup(repoPath):
    return f'''
    rm -rf {os.path.join(repoPath, 'dump')}
    '''


def copyMongoDbBackupFiles(containerName, repoPath):
    return f'''
    docker cp {containerName}:/backup/ {os.path.join(repoPath, 'dump')}
    '''


################################################ -------------------------------################################################
# [3/4] Fs

def removeFsBackupFiles(repoPath):
    return f'''
    rm -rf {os.path.join(repoPath, 'dump')}
    '''


def copyFsBackupFiles(srcPath, repoPath):
    return f'''
    cp -r {srcPath} {os.path.join(repoPath, 'dump')}
    '''

################################################ -------------------------------################################################
# [4/4] DS

def removeDsBackupFiles(repoPath):
    return f'''
    rm -rf {os.path.join(repoPath, 'dump')}
    '''


def copyDsBackupFiles(srcPath, repoPath):
    return f'''
    cp -r {srcPath} {os.path.join(repoPath, 'dump')}
    '''



from .engines import mysqlDefault
from .engines import mysqlDocker
from .engines import mongoDocker
from .engines import fsDefault
from .engines import dsDefault
from .engines import dsDefault2
from .engines import svrDefault
from .engines import vsDefault
# Entry Function


def start(config):
    DB_ENGINE = config.DB_ENGINE
    MsqModule = ''

    if (DB_ENGINE == 'mysql'):
        MsqModule = mysqlDefault
    elif (DB_ENGINE == 'mysqlDocker'):
        MsqModule = mysqlDocker
    elif (DB_ENGINE == 'mongoDocker'):
        MsqModule = mongoDocker
    elif (DB_ENGINE == 'fsDefault'):
        MsqModule = fsDefault
    elif (DB_ENGINE == 'dsDefault'):
        MsqModule = dsDefault
    elif (DB_ENGINE == 'dsDefault2'):
        MsqModule = dsDefault2
    elif (DB_ENGINE == 'svrDefault'):
        MsqModule = svrDefault
    elif (DB_ENGINE == 'vsDefault'):
        MsqModule = vsDefault
    else:
        MsqModule = mysqlDefault

    MsqModule.start(config)


# Example Call Below ***-----------***-----------***-----------***-----------***
class Config:
    DB_ENGINE = 'mysql'
    STAGE_STORAGE_PATH = '/home/un4/Reponere/Drive' + '/mysqlBackup/'
    GIT_PATH = 'GIT_PATH_HERE'
    USERNAME = 'un4'
    GIT_NAME = 'GIT_NAME_HERE'
    GIT_EMAIL = 'GIT_EMAIL_HERE'
    INTERVAL = 60*15  # seconds


# config = Config()
# dbbkp.main.start(config)
# NOTE: Run As/With sudo



import time
import utilum
from . import scripts
from . import formatter


def gitConfig(name, email):
    utilum.system.shell(f'git config --global user.name {name}')
    utilum.system.shell(f'git config --global user.email {email}')


def grantPermissionFolder(dbPath):
    pCmd = "chmod 777 -R " + dbPath
    utilum.system.shell(pCmd)


def gitInitOrRegular(username, repoPath, gitPath):

    def wrapperRegular(cmd):
        return f'''su {username} -c "cd {repoPath} && {cmd}"'''

    cmd1 = wrapperRegular(f'''git init --initial-branch=main''')
    cmd2 = wrapperRegular(f'''git remote add origin {gitPath}''')
    cmd3 = wrapperRegular(f'''git add .''')
    cmd4 = wrapperRegular(f'''git commit -m "Regular Update"''')
    cmd5 = wrapperRegular(f'''git push origin main''')

    grantPermissionFolder(repoPath)

    try:
        utilum.system.shell(cmd1)
    except:
        print("")

    try:
        utilum.system.shell(cmd2)
    except:
        print("")

    utilum.system.shell(cmd3)

    utilum.system.shell(cmd4)

    utilum.system.shell(cmd5)

    return None


def grantPermissionFile(dbPath):
    def wrapperAdmin(cmd):
        return f'''sudo {cmd}'''
    pCmd = wrapperAdmin("chmod 777 " + dbPath)
    utilum.system.shell(pCmd)


def manageDatabases(username, repoPath):
    def wrapperAdmin(cmd):
        return f'''{cmd}'''

    cmd1 = wrapperAdmin(scripts.showDatabases())
    out, err = utilum.system.shellRead(cmd1)
    decoded = out.decode('utf-8')
    dbs = decoded.split("\n")
    dbs = dbs[1:-1]
    # print(dbs)

    for database in dbs:
        # print(database)
        dbPath = repoPath + database + '.sql'
        exportCmd = wrapperAdmin(scripts.exportDatabase(database, dbPath))
        # print(exportCmd)
        utilum.system.shell(exportCmd)

        # format sql dump: skipping for now
        # if(database not in ['mysql','sys','information_schema','performance_schema'] or database in ['mysql','sys']):
        # formatter.process(fileReadPath = dbPath, fileWritePath = dbPath)

        # change to 777
        grantPermissionFile(dbPath)


def flow(config):
    # First Function to Init
    if (utilum.file.isPathExist(config.STAGE_STORAGE_PATH) == False):
        utilum.system.shell(f"mkdir {config.STAGE_STORAGE_PATH}")
        utilum.system.shell(f"chmod 777 -R {config.STAGE_STORAGE_PATH}")

    # Mid Function to Transfer DB Files
    manageDatabases(config.USERNAME, config.STAGE_STORAGE_PATH)

    # git config set
    gitConfig(config.GIT_NAME, config.GIT_EMAIL)

    # Last Function to Commit
    gitInitOrRegular(
        config.USERNAME, config.STAGE_STORAGE_PATH, config.GIT_PATH)


def start(config):
    count = 0.001
    while (True):
        print("\nCount: ", count)
        flow(config)
        time.sleep(config.INTERVAL)
        count += 0.001



import time
import os
import shutil
import utilum
# from . import scripts
from ..utils import gitlab


# def gitConfig(name, email, repoPath):
#     def wrapperRegular(cmd):
#         return f'''cd {repoPath} && {cmd}'''
#     utilum.system.shell(wrapperRegular(
#         f'git config --local user.name "{name}"'))
#     utilum.system.shell(wrapperRegular(
#         f'git config --local user.email "{email}"'))


def gitInitAndPush(dstToken, repoPath, chunkGroupRepoPath):
    rUrl = f'https://gitlab-ci-token:{dstToken}@gitlab.com/{chunkGroupRepoPath}'
    cmd1 = f'''
    cd '{repoPath}'
    git init --initial-branch=main
    git remote add origin {rUrl}
    
    git lfs install
    git lfs track "*.psd"
    git lfs track "*.mp4"
    git lfs track "*.webm"
    git lfs track "*.mkv"
    git lfs track "*.srt"
    git lfs track "*.chunk"
    git add .gitattributes
    '''
    cmd2 = f'''
    cd '{repoPath}'
    git remote set-url origin {rUrl}
    git config lfs.{rUrl}/info/lfs.locksverify true
    git add .
    git commit -m "Chunk Creation"
    git lfs push origin main
    git lfs push --all
    git push -u origin main
    git push --tags
    '''
    cmd3 = f'''
    cd '{repoPath}'
    git remote set-url origin "https://gitlab.com/{chunkGroupRepoPath}.git"
    '''
    utilum.system.shell(cmd1)
    utilum.system.shell(cmd2)
    utilum.system.shell(cmd3)
    return


def processChunk(config, chunk, uviPath, uniqueVideoIdPath):
    dstChunkFolderPath = os.path.join(uniqueVideoIdPath, chunk)
    dstChunkFilePath = os.path.join(dstChunkFolderPath, chunk) + ".chunk"
    gitChunkFolderPath = os.path.join(dstChunkFolderPath, ".git")
    if (utilum.file.isPathExist(dstChunkFolderPath) == False):
        utilum.file.createFolder(dstChunkFolderPath)

    srcChunkFilePath = os.path.join(uviPath, chunk)
    if (utilum.file.isPathExist(dstChunkFilePath) == False):
        shutil.copy2(srcChunkFilePath, dstChunkFilePath)

    uniqueVideoGroup = config.uniqueVideoGroup
    cp = gitlab.createProject({
        "name": chunk,
        "path": chunk,
        "visibility": 'private',  # !important
        "description": f'Chunk Number: {chunk}',
    }, uniqueVideoGroup["id"], config.GIT_TOKEN)
    # print("cp: ", cp)  # :debug

    chunkGroupRepoPath = config.chunkGroupRepoPath
    if (utilum.file.isPathExist(gitChunkFolderPath)):
        # [1] git repo exist
        # copy the file and push, for accounting the chance of change in chunk
        gitInitAndPush(config.GIT_TOKEN, dstChunkFolderPath,
                       chunkGroupRepoPath)
    else:
        # [2] initialize new git repo
        # init a new git-lfs and push
        gitInitAndPush(config.GIT_TOKEN, dstChunkFolderPath,
                       chunkGroupRepoPath)
    return


def processUniqueVideoId(config, uniqueVideoId, sourceEntityPath, dstEntityPath):
    rootGroup = config.rootGroup
    jsonData = {"name": uniqueVideoId,
                "path": uniqueVideoId,
                "visibility": 'private',  # !important
                "parent_id": rootGroup["id"]}
    uniqueVideoGroupResponse = gitlab.createGroup(jsonData, config.GIT_TOKEN)
    # print("uniqueVideoGroupResponse: ", uniqueVideoGroupResponse)  # :debug

    uniqueVideoRootGroupPath = os.path.join(
        config.rootGroupPath, uniqueVideoId)
    searchPath = os.path.join(config.rootGroup["full_path"], uniqueVideoId)
    uniqueVideoGroup = gitlab.getGroup(
        config.GIT_TOKEN, searchPath, uniqueVideoRootGroupPath)
    # print("uniqueVideoGroup: ", uniqueVideoGroup)  # :debug
    config.uniqueVideoGroup = uniqueVideoGroup

    uniqueVideoIdPath = os.path.join(dstEntityPath, uniqueVideoId)
    if (utilum.file.isPathExist(uniqueVideoIdPath) == False):
        utilum.file.createFolder(uniqueVideoIdPath)

    uviPath = os.path.join(sourceEntityPath, uniqueVideoId)
    chunks = os.listdir(uviPath)
    for chunk in chunks:
        chunkGroupRepoPath = os.path.join(config.uniqueVideoGroupPath, chunk)
        config.chunkGroupRepoPath = chunkGroupRepoPath
        processChunk(config, chunk, uviPath, uniqueVideoIdPath)
    # print("chunks: ", chunks)  # :debug
    return


def processRoot(config, path):
    allowedPaths = ['all', 'api', 'ui']
    if (path not in allowedPaths):
        return
    jsonData = {"name": path,
                "path": path,
                "visibility": 'private',  # !important
                "parent_id": config.GIT_GROUP_ID}
    rootGroupResponse = gitlab.createGroup(jsonData, config.GIT_TOKEN)
    # print("rootGroupResponse: ", rootGroupResponse)  # :debug
    rootGroupPath = os.path.join(config.GIT_GROUP_PATH, path)
    config.rootGroupPath = rootGroupPath

    searchPath = os.path.join(config.GIT_GROUP_PATH.split("/")[-1], path)
    rootGroup = gitlab.getGroup(config.GIT_TOKEN, searchPath, rootGroupPath)
    # print("rootGroup: ", searchPath, rootGroup)  # :debug
    config.rootGroup = rootGroup

    dstEntityPath = os.path.join(config.DST_PATH, path)
    if (utilum.file.isPathExist(dstEntityPath) == False):
        utilum.file.createFolder(dstEntityPath)

    sourceEntityPath = os.path.join(config.SRC_PATH, path)
    uniqueVideoIds = os.listdir(sourceEntityPath)
    # print("uniqueVideoIds: ", uniqueVideoIds)  # :debug
    for uniqueVideoId in uniqueVideoIds:
        uniqueVideoGroupPath = os.path.join(rootGroupPath, uniqueVideoId)
        config.uniqueVideoGroupPath = uniqueVideoGroupPath
        processUniqueVideoId(config, uniqueVideoId,
                             sourceEntityPath, dstEntityPath)


def flow(config):
    # [1] all, process root
    processRoot(config, 'all')
    
    # [2] ui, process root
    processRoot(config, 'ui')
    
    # [3] api, process root
    processRoot(config, 'api')


def start(config):
    count = 0.001

    if (utilum.file.isPathExist(config.DST_PATH) == False):
        utilum.file.createFolder(config.DST_PATH)

    GIT_TOKEN = utilum.file.readFile(config.GIT_TOKEN_PATH).replace("\n", "")
    config.GIT_TOKEN = GIT_TOKEN

    while (True):
        print("\nCount: ", count)
        flow(config)
        time.sleep(config.INTERVAL)
        count += 0.001



import time
import utilum
from . import scripts


def gitConfig(name, email, repoPath):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''
    utilum.system.shell(wrapperRegular(
        f'git config --local user.name "{name}"'))
    utilum.system.shell(wrapperRegular(
        f'git config --local user.email "{email}"'))


def gitInitOrRegular(repoPath, message, branch):
    def wrapperRegular(cmd):
        return f'''cd {repoPath} && {cmd}'''

    cmd3 = wrapperRegular(f'''git add .''')
    cmd4 = wrapperRegular(f'''git commit -m "{message}"''')
    cmd5 = wrapperRegular(f'''git push origin {branch}''')

    utilum.system.shell(cmd3)
    utilum.system.shell(cmd4)
    utilum.system.shell(cmd5)

    return None






import utilum
from sql_formatter.core import format_sql


# fileReadPath = "input.sql"
# fileWritePath = "output.sql"
fw1 = "output1.sql"
fw2 = "output2.sql"

class Config:
    language= "spark", # Defaults to "sql" (see the above list of supported dialects)
    indent= " ", # Defaults to two spaces
    uppercase= True, # Defaults to false
    linesBetweenQueries= 2, # Defaults to 1

config = Config()
DEV_MODE = False


def process(fileReadPath, fileWritePath):
  # print("<formatter.process>")
  originalOutput = utilum.file.readFile(fileReadPath)
  formattedOutput = originalOutput;
  formattedFinalOutput = formattedOutput.replace("),(", "),\n(").replace(");",")\n;").replace("),\n(",")\n,(")
  # .replace(",\n  ", ",").replace("\n;", ";").replace(";\n", ";").replace("(\n", "(").replace("\n )", ")")

  o1 = format_sql(originalOutput)
  o2 = format_sql(formattedOutput)

  if (o1 == o2):
      utilum.file.clearFile(fileWritePath)
      utilum.file.writeFile(fileWritePath, formattedFinalOutput)
  else:
    if (DEV_MODE):
      utilum.file.clearFile(fw1)
      utilum.file.writeFile(fw1, o1)
      utilum.file.clearFile(fw2)
      utilum.file.writeFile(fw2, o2)
    # pass
    # No Re-write since original file is not same formatted file
  # print("</formatter.process>")


import os
import requests

null = None
true = True
false = False
GITLAB_BASE_URL = 'https://gitlab.com/api/v4'
GROUPS = 101
SUB_GROUPS = 101

def getFileSize(filePath):
    return os.path.getsize(filePath)


def parseJson(jsonString):
    obj = eval(jsonString)
    return obj


def getProject(projectId, token):
    project = requests.get(f'''{GITLAB_BASE_URL}/projects/{str(projectId)}?statistics=true''', headers={"PRIVATE-TOKEN": token})
    projectT = eval(project.text)
    return projectT


def getGroupProjects(groupId, projectSearch, page, pageSize, token):
    projects = requests.get(f'''{GITLAB_BASE_URL}/groups/{str(groupId)}/projects?search={projectSearch}&include_subgroups=true&owned=true&per_page={str(pageSize)}&page={str(page)}''', headers={"PRIVATE-TOKEN": token})
    projects = eval(projects.text)
    return projects


def getGroups(token):
    groups = requests.get(f'''{GITLAB_BASE_URL}/groups/?per_page={str(GROUPS)}&top_level_only=true''', headers={"PRIVATE-TOKEN": token})
    groupsT = eval(groups.text)
    return groupsT

def getGroup(token, search, fullPath):
    groups = requests.get(f'''{GITLAB_BASE_URL}/groups/?per_page={str(GROUPS)}&search={search}''', headers={"PRIVATE-TOKEN": token})
    groupsT = eval(groups.text)
    if(len(groupsT) >= 1 and (type('stream') != type(groupsT))):
        # print("groupsT: ", groupsT) # :debug
        for gr in groupsT:
            # print("gr: ", gr) # :debug
            fp = gr["full_path"].lower()
            # print("fp-l: ", fp) # :debug
            # print("fp-r: ", fullPath.lower()) # :debug
            if(fp == fullPath.lower()):
                return gr
        return {}
    else:
        return {}

def getSubGroups(groupId, token):
    subGroups = requests.get(f'''{GITLAB_BASE_URL}/groups/{str(groupId)}/subgroups?per_page={str(SUB_GROUPS)}''', headers={"PRIVATE-TOKEN": token})
    subGroups = eval(subGroups.text)
    return subGroups


def createGroup(json, token):
    subGroups = requests.post(f'''{GITLAB_BASE_URL}/groups/''', json=json, headers={"PRIVATE-TOKEN": token})
    subGroups = eval(subGroups.text)
    return subGroups
# {'message': 'Failed to save group {:path=>["can contain only letters, digits, \'_\', \'-\' and \'.\'. Cannot start with \'-\' or end in \'.\', \'.git\' or \'.atom\'."]}'}    

def createProject(json, namespace_id, token):
    project = requests.post(f'''{GITLAB_BASE_URL}/projects/?namespace_id={namespace_id}''', json=json, headers={"PRIVATE-TOKEN": token})
    project = eval(project.text)
    return project
